using System;
using System.ComponentModel;
using System.Windows.Forms;
using Flibs;

namespace WinTestRunner {
    public class FlibViewer : Form {
        private readonly Flib _flib;
        private readonly ExperimentData _experimentData;
        private RichTextBox _richTextBox;
        private Container components = null;

        public FlibViewer(Flib flib, ExperimentData experimentData) {
            _flib = flib;
            _experimentData = experimentData;
            InitializeComponent();
        }

        protected override void Dispose(bool disposing) {
            if (disposing) {
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this._richTextBox = new System.Windows.Forms.RichTextBox();
            this.SuspendLayout();
            // 
            // _richTextBox
            // 
            this._richTextBox.Dock = System.Windows.Forms.DockStyle.Fill;
            this._richTextBox.Font = new System.Drawing.Font("Courier New", 12F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
            this._richTextBox.Location = new System.Drawing.Point(0, 0);
            this._richTextBox.Name = "_richTextBox";
            this._richTextBox.ReadOnly = true;
            this._richTextBox.Size = new System.Drawing.Size(672, 502);
            this._richTextBox.TabIndex = 0;
            this._richTextBox.Text = "";
            this._richTextBox.WordWrap = false;
            // 
            // FlibViewer
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(672, 502);
            this.Controls.Add(this._richTextBox);
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "FlibViewer";
            this.ShowInTaskbar = false;
            this.Text = "Flib Viewer";
            this.Load += new System.EventHandler(this.FlibViewer_Load);
            this.ResumeLayout(false);

        }

        #endregion

        private string Format(int num, int length) {
            string result = num.ToString();

            if (result.Length < length)
                result = new string(' ', length - result.Length) + result;

            return result;
        }

        private void FlibViewer_Load(object sender, EventArgs e) {
            string text = string.Format("  {0}\n   ", _experimentData._count);
            text += string.Format("  {0}\n   ", _experimentData._best);
            text += string.Format("  {0}\n   ", _experimentData._average);
            text += string.Format("  {0}\n   ", _experimentData._worst);

            for (int i = 0; i < Branch.TARGET_COUNT; i++) {
                text += Format(i, 10);
            }
            text += "\n";

            for (int i = 0; i < _flib.States.Length; i++) {
                text += Format(i, 5);
                foreach (Branch branch in _flib.States[i].Branches) {
                    text += Format(branch.StateIndex, 6) + Format(branch.Output, 4);
                }
                text += "\n";
            }
            
            _richTextBox.Text = text;
        }
    }
}