using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using Flibs;
using NPlot;
using PlotSurface2D = NPlot.Windows.PlotSurface2D;

namespace WinTestRunner {
    public class OutputForm : Form {
        private delegate void RedrawPlotEventHandler(ArrayList x, ArrayList averageBest, ArrayList averageWorst, ArrayList averageAverage,
                                                     Flib bestFoundFlib);

        private Button _okButton;
        private Container components = null;
        private PlotSurface2D _plotSurface2D;
        private Button _bestPredictorButton;
        private ExecuteData _executeData;
        private Flib _bestFoundFlib;
        private Thread _calculateThread;
        private ExperimentData _experimentData = new ExperimentData();
        private System.Windows.Forms.Button _savePlotButton;
        private double _totalFitness = 0;
        private PlotData _plotData;

        private OutputForm() {
            InitializeComponent();
        }

        public OutputForm(ExecuteData executeData) : this() {
            _executeData = executeData;
        }

        protected override void Dispose(bool disposing) {
            if (disposing) {
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent() {
            this._okButton = new System.Windows.Forms.Button();
            this._plotSurface2D = new NPlot.Windows.PlotSurface2D();
            this._bestPredictorButton = new System.Windows.Forms.Button();
            this._savePlotButton = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // _okButton
            // 
            this._okButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this._okButton.Location = new System.Drawing.Point(448, 328);
            this._okButton.Name = "_okButton";
            this._okButton.TabIndex = 0;
            this._okButton.Text = "OK";
            this._okButton.Click += new System.EventHandler(this._okButton_Click);
            // 
            // _plotSurface2D
            // 
            this._plotSurface2D.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
                | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right)));
            this._plotSurface2D.AutoScaleAutoGeneratedAxes = true;
            this._plotSurface2D.AutoScaleTitle = true;
            this._plotSurface2D.BackColor = System.Drawing.SystemColors.ControlLightLight;
            this._plotSurface2D.DateTimeToolTip = false;
            this._plotSurface2D.Legend = null;
            this._plotSurface2D.LegendZOrder = -1;
            this._plotSurface2D.Location = new System.Drawing.Point(8, 8);
            this._plotSurface2D.Name = "_plotSurface2D";
            this._plotSurface2D.Padding = 10;
            this._plotSurface2D.RightMenu = null;
            this._plotSurface2D.ShowCoordinates = true;
            this._plotSurface2D.Size = new System.Drawing.Size(512, 312);
            this._plotSurface2D.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;
            this._plotSurface2D.TabIndex = 1;
            this._plotSurface2D.Title = "";
            this._plotSurface2D.TitleFont = new System.Drawing.Font("Arial", 14F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Pixel);
            this._plotSurface2D.XAxis1 = null;
            this._plotSurface2D.XAxis2 = null;
            this._plotSurface2D.YAxis1 = null;
            this._plotSurface2D.YAxis2 = null;
            // 
            // _bestPredictorButton
            // 
            this._bestPredictorButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this._bestPredictorButton.Enabled = false;
            this._bestPredictorButton.Location = new System.Drawing.Point(296, 328);
            this._bestPredictorButton.Name = "_bestPredictorButton";
            this._bestPredictorButton.Size = new System.Drawing.Size(144, 23);
            this._bestPredictorButton.TabIndex = 2;
            this._bestPredictorButton.Text = " ";
            this._bestPredictorButton.Click += new System.EventHandler(this._bestPredictorButton_Click);
            // 
            // _savePlotButton
            // 
            this._savePlotButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
            this._savePlotButton.Enabled = false;
            this._savePlotButton.Location = new System.Drawing.Point(152, 328);
            this._savePlotButton.Name = "_savePlotButton";
            this._savePlotButton.Size = new System.Drawing.Size(136, 23);
            this._savePlotButton.TabIndex = 3;
            this._savePlotButton.Text = " ";
            this._savePlotButton.Click += new System.EventHandler(this._savePlotButton_Click);
            // 
            // OutputForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(528, 358);
            this.Controls.Add(this._savePlotButton);
            this.Controls.Add(this._bestPredictorButton);
            this.Controls.Add(this._plotSurface2D);
            this.Controls.Add(this._okButton);
            this.Name = "OutputForm";
            this.ShowInTaskbar = false;
            this.Text = "0";
            this.Load += new System.EventHandler(this.OutputForm_Load);
            this.Closed += new System.EventHandler(this.OutputForm_Closed);
            this.ResumeLayout(false);

        }

        #endregion

        private void _okButton_Click(object sender, EventArgs e) {
            Close();
        }

        private void OutputForm_Load(object sender, EventArgs e) {
            _calculateThread = new Thread(new ThreadStart(Calculate));
            _calculateThread.Start();
        }

        private void Calculate() {
            ArrayList worst = new ArrayList();
            ArrayList best = new ArrayList();
            ArrayList average = new ArrayList();

            for (int i = 1; i <= _executeData.PassCount; i++) {
                Engine engine = new Engine(_executeData);
                ArrayList x = new ArrayList();
                ArrayList worstFitness = new ArrayList();
                ArrayList bestFitness = new ArrayList();
                ArrayList averageFitness = new ArrayList();

                ExecuteStep(engine, x, bestFitness, worstFitness, averageFitness);

                AddArray(worst, worstFitness);
                AddArray(best, bestFitness);
                AddArray(average, averageFitness);

                lock (this) {
                    if (!IsHandleCreated)
                        return;

                    BeginInvoke(new RedrawPlotEventHandler(UpdateData),
                                new object[] {x, GetAverage(best, i), GetAverage(worst, i), GetAverage(average, i), engine.BestFoundFlib});
                }
            }
        }

        private void UpdateData(ArrayList x, ArrayList averageBest, ArrayList averageWorst, ArrayList averageAverage,
                                Flib bestFoundFlib) {
            if(_bestFoundFlib == null) {
                _bestFoundFlib = bestFoundFlib;
                _experimentData._best = bestFoundFlib.Fitness;
                _experimentData._worst = bestFoundFlib.Fitness;
            }
            
            if(_experimentData._best < bestFoundFlib.Fitness)
                _experimentData._best = bestFoundFlib.Fitness;

            if(_experimentData._worst > bestFoundFlib.Fitness)
                _experimentData._worst = bestFoundFlib.Fitness;

            if (_bestFoundFlib.Fitness < bestFoundFlib.Fitness)
                _bestFoundFlib = bestFoundFlib;

            _totalFitness += bestFoundFlib.Fitness;
            _experimentData._count++;
            _experimentData._average = _totalFitness/_experimentData._count;

            Text = _experimentData._count.ToString();

            _bestPredictorButton.Enabled = _bestFoundFlib != null;

            _plotSurface2D.Clear();

            _plotSurface2D.Add(AddPlot(averageBest, x, " ", Color.Green));
            _plotSurface2D.Add(AddPlot(averageWorst, x, " ", Color.Blue));
            _plotSurface2D.Add(AddPlot(averageAverage, x, " ", Color.Red));

            _plotSurface2D.Refresh();

            if(_experimentData._count == _executeData.PassCount) {
                _plotData = new PlotData(x, averageBest);
                _savePlotButton.Enabled = true;
            }
        }

        private void AddArray(ArrayList array, ArrayList addition) {
            if (array.Count == 0)
                array.AddRange(addition);
            else {
                for (int i = 0; i < array.Count; i++) {
                    array[i] = (float) array[i] + (float) addition[i];
                }
            }
        }

        public ArrayList GetAverage(ArrayList array, int count) {
            ArrayList result = new ArrayList(array.Count);
            foreach (float item in array) {
                result.Add(item/count);
            }
            return result;
        }

        private void ExecuteStep(Engine engine, ArrayList x, ArrayList bestFitness, ArrayList worstFitness, ArrayList averageFitness) {
            while (engine.ExecuteStep()) {
                x.Add((float) (x.Count + 1));
                bestFitness.Add((float) engine.BestFitness);
                worstFitness.Add((float) engine.WorstFitnes);
                averageFitness.Add((float) engine.AverageFitness);
            }
        }

        private LinePlot AddPlot(ArrayList y, ArrayList x, string label, Color color) {
            LinePlot plot = new LinePlot(y, x);
            plot.Label = label;
            plot.Color = color;
            return plot;
        }

        private void _bestPredictorButton_Click(object sender, EventArgs e) {
            new FlibViewer(_bestFoundFlib, _experimentData).ShowDialog();
        }

        private void OutputForm_Closed(object sender, System.EventArgs e) {
            if(_calculateThread.IsAlive)
                _calculateThread.Abort();
        }

        private void _savePlotButton_Click(object sender, System.EventArgs e) {
            SaveFileDialog fileDialog = new SaveFileDialog();
            if(fileDialog.ShowDialog() == DialogResult.OK) {
                _plotData.Save(fileDialog.FileName);
            }
        }
    }
}